from trac.core import *
from trac.util import escape, Markup
from trac.wiki.api import parse_args
from trac.wiki.macros import WikiMacroBase
from trac.wiki.formatter import wiki_to_html
from trac.util import format_datetime
from StringIO import StringIO


class ChangeLogMacro(WikiMacroBase):
    """ Provides the macro

    {{{
       [[ChangeLog(paths[,limit[,rev]])]]
    }}}

    which dumps the change log for paths of revision rev, back
    limit revisions. "rev" can be 0 for the latest revision.
    "paths" is a vertical bar separated list of SVN paths.

    limit and rev may be keyword arguments
    """

    def expand_macro(self, formatter, name, content):
        req = formatter.req
        args, kwargs = parse_args(content)
        args += [None, None]
        paths, limit, rev = args[:3]
        limit = kwargs.pop('limit', limit)
        rev = kwargs.pop('rev', rev)

        if 'CHANGESET_VIEW' not in req.perm:
            return Markup('<i>Changelog not available</i>')

        repo = self.env.get_repository(req.authname)

        if rev is None:
            rev = repo.get_youngest_rev()
        rev = repo.normalize_rev(rev)
        if limit is None:
            limit = 5
        else:
            limit = int(limit)

	changeSets = {}

	pathList = paths.split('|');
	for path in pathList:
	    path = repo.normalize_path(path)

	    node = repo.get_node(path, rev)
	    for npath, nrev, log in node.get_history(limit):
		change = repo.get_changeset(nrev)
		changeSets[nrev] = [change.author, format_datetime(change.date), change.message]
	
        out = StringIO()
        out.write('<div class="changelog">\n')
	cnt = 0
	for nrev, changeList in sorted(changeSets.iteritems(),None,None,True):
	    if cnt >= limit:
		break
	    cnt = cnt + 1
	    change_author, change_date, change_message = changeList
	    out.write(wiki_to_html("'''[%i] by %s on %s'''\n\n%s" % (nrev, change_author, change_date, change_message),
                                   self.env, req));
        out.write('</div>\n')
        return out.getvalue()
